<?php

namespace FiloBlu\Esb\Core\Expression;

use Magento\Framework\DataObject;
use RuntimeException;

/**
 * Class Expression
 * @package FiloBlu\Esb\Core\Expression
 */
class Expression implements ExpressionInterface
{
    /**
     * @var string
     */
    protected $field;
    /**
     * @var string
     */
    protected $operator;
    /**
     * @var string
     */
    protected $value;

    /**
     * @param DataObject $data
     * @return int
     */
    public function evaluate(DataObject $data): int
    {
        $leftValue = $this->extractValue($this->field, $data);
        $rightValue = $this->extractValue($this->value, $data);

        return $this->operate($leftValue, $rightValue);
    }

    /**
     * @param $value
     * @param DataObject $data
     * @return mixed
     */
    public function extractValue($value, DataObject $data)
    {
        if ($value[0] !== '{') {
            return $value;
        }

        $key = trim($value, '{}');

      /*  if (!$data->hasData()) {
            throw  new RuntimeException("Invalid expression {$value}");
        }*/

        return $data->getDataByPath($key);
    }

    /**
     * @param $leftValue
     * @param $rightValue
     * @return bool
     */
    public function operate($leftValue, $rightValue)
    {

        // TODO : must be configurable vi di.xml
        switch ($this->getOperator()) {
            case 'nin':
                return !in_array($leftValue, is_array($rightValue) ? $rightValue : [$rightValue], false);
            case 'in':
                return in_array($leftValue, is_array($rightValue) ? $rightValue : [$rightValue], false);
            case 'eq':
                return $leftValue == $rightValue;
            case 'seq':
                return $leftValue === $rightValue;
            case 'lt':
                return $leftValue < $rightValue;
            case 'lte':
                return $leftValue <= $rightValue;
            case 'gt':
                return $leftValue > $rightValue;
            case 'gte':
                return $leftValue >= $rightValue;
            case 'neq':
                return $leftValue != $rightValue;

        }
    }

    /**
     * @return mixed
     */
    public function getOperator()
    {
        return $this->operator;
    }

    /**
     * @param $operator
     * @return $this
     */
    public function setOperator($operator): ExpressionInterface
    {
        $this->operator = $operator;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getField()
    {
        return $this->field;
    }

    /**
     * @param $field
     * @return $this
     */
    public function setField($field): ExpressionInterface
    {
        $this->field = $field;
        return $this;
    }

    /**
     * @return mixed
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param $value
     * @return $this
     */
    public function setValue($value): ExpressionInterface
    {
        $this->value = $value;
        return $this;
    }
}
